"""
CSV2Notion Neo - Main Entry Point

This module serves as the main entry point for the CSV2Notion Neo application.
It provides a simple interface to launch the CLI application.
"""

from csv2notion_neo.cli import main

if __name__ == "__main__":  # pragma: no cover
    main()
